#  Copyright (c) 2006 Six Feet Up, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
An ad for use with the Ad Manager
"""
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import VariableSchemaSupport
from Products.Archetypes.public import DisplayList
from Products.Archetypes.public import BaseSchema, Schema
from Products.Archetypes.public import BaseContent, registerType
from Products.Archetypes.public import TextField, ImageField, ReferenceField, \
    StringField
from Products.Archetypes.public import TextAreaWidget, ImageWidget, StringWidget, \
    SelectionWidget
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import ReferenceBrowserWidget
from config import PROJECTNAME

NewSchema = BaseSchema.copy()
for fieldname in ('title',):
   NewSchema[fieldname].required = 0

schema = NewSchema + Schema((
    StringField(
        'title',
        widget=StringWidget(
            label = 'Ad Title',
            description = 'This is the title that shows up in the ad.',
        ),
    ),
    ReferenceField(
        'adLocation',
        multiValued = 1,
        relationship='AdLocation',
        widget=ReferenceBrowserWidget(
            label='Ad Location Step 1',
            description="""Select where the ad should show up""",
            startup_directory='/',
        ),
    ),
    StringField(
        'adSlot',
        vocabulary = 'getSlotVocab',
        index='portal_ad_catalog/FieldIndex',
        widget=SelectionWidget(
            label='Ad Location Step 2',
            description="""Select an optional slot to use""",
            ),
    ),
    TextField(
        'adText',
        default_output_type = 'text/html',
        allowable_content_types = ('text/plain',
                                   'text/structured',
                                   'text/html'
                                   ),
        widget=TextAreaWidget(
            label='Ad Text',
            description='Add text to your ad'
        ),
    ),
    ReferenceField(
        'adLinkInternal',
        multiValued = 0,
        relationship='InternalLink',
        widget=ReferenceBrowserWidget(
            label='Internal link',
            description='Select where this ad is linked to',
            startup_directory='/',
        ),
    ),
    StringField(
        'adLinkExternal',
        widget=StringWidget(
            label = 'External link',
            description = 'Type in a link i.e. http://www.plone.org .  NOTE: This link will be used only if the internal link has not been selected.',
        ),
    ),
    StringField(
        'adLinkTitle',
        widget=StringWidget(
            label = 'Link Title',
            description = 'Type in the title of link',
        ),
    ),
    ImageField(
        'adImageTop',
        widget=ImageWidget(
            label='Top Ad Image',
            description='Add an image to the top of your ad',
        ),
    ),
    ImageField(
        'adImageBottom',
        widget=ImageWidget(
            label='Bottom Ad Image',
            description='Add an image to the bottom of your ad',
        ),
    ),
))

class Ad(BaseContent, VariableSchemaSupport):
    """Create an ad
    """
    schema = schema
    global_allow = 0
    content_icon = 'ad_icon.gif'
    
    # TODO: shouldn't have to do this
    def Title(self):
        """get the title
        """
        return self.getTitle()

    def getSlotVocab(self):
        ad_tool = getToolByName( self, 'portal_ad_manager' )
        ad_slots = []
        raw_ad_slots = ad_tool.getSlots()
        for slot in raw_ad_slots:
            data = (slot , slot)
            ad_slots.append(data)
        return DisplayList(ad_slots)
    
    def getSchema(self):
        ad_tool = getToolByName( self, 'portal_ad_manager' )
        newschema = self.schema
        #get the allowed types
        ad_locations = ad_tool.getAdLocations()
        ad_links = ad_tool.getAdLinks()
        # get the size for images from the folder above
        o_size = ()
        folder = self.getParentNode()
        width = folder.getWidth()
        height = folder.getHeight()
        if width and height:
            o_size = (width ,height )
        # set the original size for the images
        for fieldname in ('adImageTop','adImageBottom'):
            newschema[fieldname].original_size = o_size
            newschema[fieldname].widget.description = 'Add a ' + str(width) + 'px wide image.  NOTE: Wider images will be resized to a width of ' + str(width) + 'px'
        # set the allowed types for links and location
        newschema['adLocation'].allowed_types = ad_locations
        newschema['adLinkInternal'].allowed_types = ad_links
        
        return self.schema+newschema
    
    actions = (
        { 'id': 'preview',
          'name': 'Ad Preview',
          'action': 'string:${object_url}/ad_preview',
          'permissions': (CMFCorePermissions.ModifyPortalContent,),
          'category':'object',
        },
    )
           
registerType(Ad, PROJECTNAME)
