#  Copyright (c) 2006 Six Feet Up, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
The Ad Manager that takes care of showing and storing ads
"""
import random
import sys
from AccessControl import ClassSecurityInfo
from Products.ATContentTypes.content.folder import ATFolder
from Products.ATContentTypes.content.folder import ATFolderSchema
from Products.Archetypes.public import Schema
from Products.Archetypes.public import registerType
from Products.Archetypes.public import LinesField
from Products.Archetypes.public import LinesWidget
from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.CMFCore.CMFCorePermissions import ManagePortal
from ZTUtils import LazyFilter
from config import PROJECTNAME

_marker = []
    
class AdManager(ATFolder,UniqueObject):
    """Used to manage ads
    """
    security = ClassSecurityInfo()
    archetype_name = portal_type = meta_type = 'Ad Manager'
    allowed_content_types = ['Ad Folder']
    suppl_views    = ('folder_summary_view','folder_tabular_view','folder_listing')
    content_icon   = 'billboard_icon.gif'

    NewSchema = ATFolderSchema.copy()
    NewSchema['id'].widget.visible = {'view' : 'hidden', 'edit' : 'hidden'}

    schema = NewSchema + Schema((
        LinesField(
            'slots',
            default=['Select a slot',
                     'Slot 1',
                     'Slot 2',
                     'Slot 3'
                     ],
            widget=LinesWidget(
                label='Slots for ads',
                description='Set up the list of slots for the ads.  If you add slots then you will need to add a new portlet and modify the slot call.',
                ),
        ),
    ))

    manage_options = ( ({ 'label' : 'Configure', 'action' : 'manage_configure' }
                     ,
                     ) + ATFolder.manage_options
                     )

    security.declareProtected( ManagePortal, 'manage_configure' )
    manage_configure = PageTemplateFile('www/configureAdManager', globals(),
                                   __name__='manage_configure')

    # tool-constructors have no id argument, the id is fixed
    def __init__(self):
        ATFolder.__init__(self,'portal_ad_manager')
        self.ad_locations = []
        self.ad_links = []
    
    security.declareProtected( ManagePortal, 'edit_configuration' )
    def edit_configuration(self, use_ad_locations=[], use_ad_links=[]):
        """ Change the configuration of the ad manager tool """
        self.ad_locations = use_ad_locations
        self.ad_links = use_ad_links
        if hasattr(self.REQUEST, 'RESPONSE'):
            self.REQUEST.RESPONSE.redirect('manage_configure')

    security.declarePublic('getAdLocations')
    def getAdLocations(self):
        """ Returns a list of types where ads can be located
        """
        return self.ad_locations
        
    security.declarePublic('getAdLinks')
    def getAdLinks(self):
        """ Returns a list of types ads can be linked to
        """
        return self.ad_links
    
    security.declareProtected('View','getAd')
    def getAd(self, object=None, slot=None, template=None):
        """Retrieve an ad from the ad manager
        """
        catalog_tool = getToolByName(self,'portal_ad_catalog')
        prop_tool = getToolByName(self, 'portal_properties')
        ad_states = prop_tool.promo_engine_properties.ad_states
        ads = []
        unique_objects = []
        unique_slots = []
        
        # get the ads based on the object's references
        if object:
            # try and except in case the object doesn't support references
            try:
                raw_object_ads = object.getBRefs('AdLocation')
                object_ads = LazyFilter(raw_object_ads, skip='View')
                wf_tool = getToolByName(self, 'portal_workflow')
                published_object_ads = [obj for obj in object_ads if wf_tool.getInfoFor(obj,'review_state',None) in ad_states]
                unique_objects = [item.UID() for item in published_object_ads]
            except AttributeError:
                # if the object doesn't support refs then set object to none
                # so we can still use slots on that object
                object = None
        
        # get the ads based the slot wanted
        if slot:
            query = {'portal_type':'Ad',
                     'getAdSlot':slot,
                     'review_state':ad_states}
            slot_ads = catalog_tool.searchResults(query)
            unique_slots = [item.UID for item in slot_ads]
        
        # see if there is a call for a slot and an object and get the
        # unique results.  Otherwise get the unique_slots or unique_objects
        if slot and object:
            ads = [item for item in unique_objects if item in unique_slots]
        elif slot:
            ads = unique_slots
        elif object:
            ads = unique_objects
            
        # if there are ads then choose one and then get the object
        if ads:
            uidtool = getToolByName(self,'uid_catalog')
            # TODO: could optionaly use some 80/20 type functionality here
            ad_uid = random.choice(ads)
            query = {'UID':ad_uid}
            ad_brains = uidtool(query)
            for brain in ad_brains:
                ad_obj = brain.getObject()
        # otherwise return and empty macro
        else:
            ad_obj = None
            macro = template.pt_macros()['empty']
            return macro

        # look for the context in the stack
        frame = sys._getframe()
        try:
            context = _marker
            while context is _marker and frame is not None:
                context = frame.f_locals.get('econtext', _marker)
                frame = frame.f_back
            if context is _marker:
                raise RuntimeError, 'Context not found'
        finally:
            del frame
            
        context.setLocal('ad', ad_obj)
        macro = template.pt_macros()['view']
        return macro

registerType(AdManager, PROJECTNAME)

