#  Copyright (c) 2006 Six Feet Up, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
from Products.CMFCore.utils import getToolByName,manage_addTool
from Products.Archetypes.public import listTypes, process_types
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.PromoEngine.config import PROJECTNAME, GLOBALS, AD_CATALOG

from StringIO import StringIO
import sys

def installTool(self, out):
    if not hasattr(self, 'portal_ad_manager'):
        self.manage_addProduct['PromoEngine'].manage_addTool('Ad Manager')
        self.portal_ad_manager.setTitle('Ad Manager')
        out.write('Added Ad Manager tool\n')

    if not hasattr(self, AD_CATALOG):
        self.manage_addProduct['PromoEngine'].manage_addTool('Ad Catalog')
        out.write('Added Ad Catalog tool\n')
        
def registerWithAdCatalog(self, out):
    attool = getToolByName(self, 'archetype_tool')
    typeslist = listTypes(PROJECTNAME)
    for portal_type in [ti['portal_type'] for ti in typeslist]:
        catalogs = [x.getId() for x in attool.getCatalogsByType(portal_type)]
        if AD_CATALOG not in catalogs:
            catalogs.append(AD_CATALOG)
            attool.setCatalogsByType(portal_type, catalogs)

def install(self):
    out = StringIO()
    
    # install the tools
    installTool(self, out)
    installTypes(self, out, listTypes(PROJECTNAME), PROJECTNAME)
    install_subskin(self, out, GLOBALS)
    
    registerWithAdCatalog(self, out)
    
    prop_tool = getToolByName(self, 'portal_properties')
    if 'promo_engine_properties' not in prop_tool.objectIds():
        prop_tool.manage_addPropertySheet('promo_engine_properties',
                'Promo Engine Properties')
        prop_tool.promo_engine_properties._setProperty('ad_states',
                                                       ['published'],
                                                       'lines',
                                                       )
    
    from Products.PromoEngine.config import STYLESHEETS
    try:
        portal_css = getToolByName(self, 'portal_css')
        for stylesheet in STYLESHEETS:
            try:
                portal_css.unregisterResource(stylesheet['id'])
            except:
                pass
            defaults = {'id': '',
            'media': 'all',
            'enabled': True}
            defaults.update(stylesheet)
            portal_css.manage_addStylesheet(**defaults)
    except:
        # No portal_css registry
        pass
    
    # install the UID metadata for the ad manager
    ad_catalog = getToolByName(self, AD_CATALOG)
    try:
        ad_catalog.addColumn('UID')
    except:
        pass
        
    # a configlet for the ad manager
    cp = getToolByName(self,'portal_controlpanel')
    cp.registerConfiglet(
        'AdManager', #id of your Tool
        'Ad Manager', # Title of your Troduct
        'string:${portal_url}/portal_ad_manager',
        'python:True', # a condition
        'Manage Portal', # access permission
        'Products', # section to which the configlet should be added
         1, # visibility
        'AdManager',
        'ad_manager.gif', # icon in control_panel
        'Ad Manager',
        None,
    )
    
    out.write("Successfully installed %s." % PROJECTNAME)
    return out.getvalue()
    
def uninstall(self):
    # uninstall the configlet
    cp = getToolByName(self,'portal_controlpanel', None)
    if cp is not None:
        cp.unregisterConfiglet('AdManager')
